import unittest

from unittest import mock

from jenkins_plugin_manager.core import (
    JenkinsCore,
    DEFAULT_JENKINS_RELEASE,
    DEFAULT_JENKINS_CORE_URL,
)


class TestJenkinsCore(unittest.TestCase):
    """Tests for the JenkinsCore class."""

    LATEST_CORE_VERSION = "2.1.190"

    def setUp(self):
        """Setup fxture for each test."""
        core_version_patch = mock.patch("jenkins_plugin_manager.core.JenkinsCore.load_latest_core_version")
        self.core_version_mock = core_version_patch.start()
        self.addCleanup(core_version_patch.stop)
        self.addCleanup(self.core_version_mock.reset_mock)
        self.core_version_mock.return_value = self.LATEST_CORE_VERSION
        self.jenkins_core = JenkinsCore()

    def test_jenkinscore_attr(self):
        """Check the varios attributes of the class."""
        # Check default attribute
        self.assertEqual(self.jenkins_core.jenkins_release, DEFAULT_JENKINS_RELEASE)
        self.assertEqual(
            self.jenkins_core.latest_core_url,
            "{0}/{1}/latestCore.txt".format(DEFAULT_JENKINS_CORE_URL, DEFAULT_JENKINS_RELEASE),
        )
        self.assertEqual(self.jenkins_core.core_version, self.LATEST_CORE_VERSION)

        # Check custom repository
        jenkins_repo = "http://pkg.jenkins.canonical.com"
        jenkins_release = "current"
        jenkins_core_url = "http://updates.jenkins.canonical.com"
        custom_jc = JenkinsCore(
            jenkins_release=jenkins_release, jenkins_repo=jenkins_repo, jenkins_core_url=jenkins_core_url
        )
        self.assertEqual(custom_jc.jenkins_release, jenkins_release)
        self.assertEqual(custom_jc.latest_core_url, "{0}/{1}/latestCore.txt".format(jenkins_core_url, jenkins_release))

        # Check exception
        jenkins_release = "custom"
        with self.assertRaises(ValueError) as cm:
            custom_jc = JenkinsCore(jenkins_release=jenkins_release)
            self.assertEqual(
                cm.exception, ValueError("Wrong jenkins_release parameter. Valid values are [current|stable]")
            )
